<?php 

if(!defined('ABSPATH')) exit;

/**
 * FDP API helper class
 */
class Firstdata_Api {
	public static $errors;
	static $binKey;
//	const firstdata_form = 'https://test.ipg-online.com/connect/gateway/processing';
	const firstdata_form = 'https://www.ipg-online.com/connect/gateway/processing';
	
	/**
	 * Redirect customer to FDP form
	 */
	static function doRedir($order) {
	    $order_data = $order->get_data();
	    $start_num = get_post_meta($order->get_id(), '_fdp_count', false);
	    $count = (empty($start_num) ? 0 : (int) $start_num) + 1;
	    update_post_meta($order->get_id(), '_fdp_count', $count);

	    $amount = $order->get_total();
	    $locale = get_locale();
	    $lang = 'en';
	    if($locale == 'pl_PL') $lang = 'pl';
	    $data = array(
	        'chargetotal' => round($amount, 2),
	        'currency' => 985,
	        'checkoutoption' => 'combinedpage',
	        'hash_algorithm' => 'HMACSHA256',
	        'language' => $lang,
	        'oid' => 'WC_' . $order->get_id() . '_' . $count,
	        'responseFailURL' => add_query_arg('a', 'd', WC()->api_request_url('Firstdata_Gateway')),
	        'responseSuccessURL' => add_query_arg('a', 'd', WC()->api_request_url('Firstdata_Gateway')),
	        'storename' => trim(get_option('firstdata_pos_id')),
	        'timezone' => 'UTC',
	        'transactionNotificationURL' => add_query_arg('a', 'i', WC()->api_request_url('Firstdata_Gateway')),
	        'txndatetime' => date("Y:m:d-H:i:s"),
	        'txntype' => 'sale',
	        'bcity' => trim($order_data['billing']['city']),
	        'bzip' => trim($order_data['billing']['postcode']),
	        //'bcountry' => trim($order_data['billing']['country']),
	        'email' => $order_data['billing']['email'],
	    );
	    ksort($data);
//	    error_log('doRedir ' . print_r($data, true));
	    ?><!DOCTYPE HTML><html <?php language_attributes() ?>><head><meta charset="<?php bloginfo('charset') ?>" /><?php wp_head() ?></head><body><br/><br/><br/><h3 style="text-align: center;"><?= __('Redirecting to FirstData...', 'firstdata') ?></h3><?php
	    ?><form action="<?= self::firstdata_form ?>" method="post" id="firstdata_payment_form"><?php
	    foreach($data as $k => $v) self::genHidden($k, $v);
	    self::genHidden('hashExtended', self::createExtendedHash($data));
        ?><script type="text/javascript">document.getElementById('firstdata_payment_form').submit();</script></form><?php wp_footer() ?></body></html><?php
	}

	/**
	 * Test integrity of data received from API in notification url
	 */
	static function validIpn() {
	    if(empty($_POST)) return 'E-30';
	    $data = array(
	        'chargetotal' => $_POST['chargetotal'],
	        'currency' => $_POST['currency'],
	        'txndatetime' => $_POST['txndatetime'],
	        'storename' => trim(get_option('firstdata_pos_id')),
	        'approval_code' => $_POST['approval_code']
	    );
	    if(!isset($_POST['notification_hash']) || $_POST['notification_hash'] != self::createExtendedHash($data)) return 'E-31';
	    $tmp = explode('_', $_POST['oid']);
	    if((count($tmp) != 3) || ($tmp[0] != 'WC')) return 'E-33';
	    $order = new WC_Order($tmp[1]);
	    if(!$order) return 'E-34';
//	    error_log('validIpn!');
	    return $data;
	}
	
	/**
	 * Test integrity of data received from API in success/fail urls
	 */
	static function validResp() {
	    if(empty($_POST)) return 'E-50';
	    $data = array(
	        'approval_code' => $_POST['approval_code'],
	        'chargetotal' => $_POST['chargetotal'],
	        'currency' => $_POST['currency'],
	        'txndatetime' => $_POST['txndatetime'],
	        'storename' => trim(get_option('firstdata_pos_id'))
	    );
	    if(!isset($_POST['response_hash']) || $_POST['response_hash'] != self::createExtendedHash($data)) return 'E-51';
	    $tmp = explode('_', $_POST['oid']);
	    if((count($tmp) != 3) || ($tmp[0] != 'WC')) return 'E-53';
	    $order = new WC_Order($tmp[1]);
	    if(!$order) return 'E-54';
//	    error_log('validResp!');
	    return $data;
	}
	
	/**
	 * Helper - convert data to hidden input field for sending as a POST request
	 */
	static function genHidden($k, $v) {
	    ?><input type="hidden" name="<?= $k ?>" value="<?= $v ?>"/><?php
	}
	
	/**
	 * Compute control hash of a given data
	 */
	static function createExtendedHash($data) {
	    $stringToHash = join('|', $data);
//	    error_log('s=' . $stringToHash . ' k=' . get_option('firstdata_key'));
	    $hash = hash_hmac('sha256', $stringToHash, get_option('firstdata_key'), true);
	    
	    return base64_encode($hash);
	}

}

?>